-module(state).
-export([main/0]).

% Simple server that can receive two types of messages:
%    {hello, From}, where From is the PID of the sender
%    {reqs, From},  where From is the PID of the sender
% If we get {hello, From}, we return the string "Hello mate!" to the sender.
% If we get {reqs, From}, we return the total requests done up until that point.
server(Reqs) ->
    receive
        {hello, From} ->
            timer:sleep(100),
            From ! "Hello mate!";
        {reqs, From} ->
            timer:sleep(100),
            From ! Reqs
    end,

    % Erlang is side-effect free, so we need to pass the new value as a
    % parameter in the recursive call.
    server(Reqs + 1).


main() ->
    Server = spawn(fun() -> server(0) end),
    Xs = lists:seq(1, 10), % Creates a list from 1 to 10
    [Server ! {hello, self()} || _ <- Xs], % Sends 10 hellos
    [receive Resp -> io:fwrite("~s~n", [Resp]) end || _ <- Xs], % Receives 10 messages
    Server ! {reqs, self()}, % Sends request to get the amount of requests
    receive Reqs -> io:fwrite("Amount of requests to server: ~p~n", [Reqs]) end,
    ok.
