-module(server_powers1).
-export([main/0]).

start_server() ->
    Pid = spawn(fun server_fun/0),
    catch unregister(server),
    register(server, Pid).

% Same server loop as in server_fun.erl
server_fun() ->
    receive
        {compute, Fun, Args, From} ->
            spawn(
              fun() ->
                      Ans = apply(Fun, Args),
                      timer:sleep(round(Ans) * 10),
                      From ! {answer, Ans}
              end)
    end,
    server_fun().


% A function to create several powers. It takes a list of numbers (Nums), whose
% elements should all be raised to the exponent Exp.
powers(Nums, Exp) ->
    % Function to create a power
    Fun = fun(X, Y) -> round(math:pow(X, Y)) end,
    % List comprehension, go through all numbers and send a message to the server
    [server ! {compute, Fun, [Num, Exp], self()} ||  Num <- Nums],
    % Receive the answers
    Ans = [receive {answer, Ans} -> Ans end || _ <- Nums],
    % Print the answers
    io:fwrite("~p ^ ~p = ~p~n", [Nums, Exp, Ans]).


main() ->
    start_server(),

    % Running the following functions yield weird results. Why is that? Because
    % we cannot guarantee the order the messages arrives in our mailbox.
    % How can this be fixed? With the use of references (see server_powers2.erl)
    powers([1,2,3,4], 3),
    powers([7,0,5,2], 3),
    powers([4,3,2,1], 3).
